import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.event.*;

/**
 * <b>Programm:</b> Wave<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Sets up all needed control UI components
 */
public class InputPanel extends JPanel
                        implements ChangeListener{
  JTextField _lambdal;
  JTextField _Al;
  JTextField _phil;
  JTextField _zooml;

  /*
   * Constructor
   * contains wave description
   */
  public InputPanel(ChangeListener cl){
    final String description = ""+
      "<HTML><BODY>"+
      "A wave is a disturbance or variation that "+
      "transfers energy progressively from point to point in a medium and that may "+
      "take the form of an elastic deformation or of a variation of pressure, electric "+
      "or magnetic intensity, electric potential, or temperature.<BR>"+
      "A wave is described by the following keys: The <B>amplitude (A)</B> which defines the "+
      "\"maximum height\", the <B>phaseshift (phi)</B> which defines where the wave starts and "+
      "the <B>wavelength (lambda)</B> which describes how the wave does look like."+
      "</BODY></HTML>";

    ManagedPanel p = new ManagedPanel();

    p.addComponent(new JLabel("lambda")  ,0,0,1,1,0,0);
    JSlider lambda = new JSlider(100,10000,3000);
    lambda.addChangeListener(cl);
    lambda.addChangeListener(this);
    lambda.setName("lambda");
    p.addComponent(lambda                ,1,0,1 ,1,0,0);
    _lambdal = new JTextField(Double.toString(lambda.getValue()/100.0),5);
    _lambdal.setEditable(false);
    p.addComponent(_lambdal              ,2,0,1 ,1,0,0);

    p.addComponent(new JLabel("A")  ,0,1,1,1,0,0);
    JSlider A = new JSlider(-300,300,100);
    A.addChangeListener(cl);
    A.addChangeListener(this);
    A.setName("A");
    p.addComponent(A                ,1,1,1,1,0,0);
    _Al = new JTextField(Double.toString(A.getValue()),5);
    _Al.setEditable(false);
    p.addComponent(_Al              ,2,1,1,1,0,0);

    p.addComponent(new JLabel("phi"),0,2,1,1,0,0);
    JSlider phi = new JSlider(-630,630,0);
    phi.addChangeListener(cl);
    phi.addChangeListener(this);
    phi.setName("phi");
    p.addComponent(phi              ,1,2,1,1,0,0);
    _phil = new JTextField(Double.toString(phi.getValue()/100.0),5);
    _phil.setEditable(false);
    p.addComponent(_phil            ,2,2,1,1,0,0);

    p.addComponent(new JLabel("Zoom"),0,3,1,1,0,0);
    JSlider zoom = new JSlider(1,500,100);
    zoom.addChangeListener(cl);
    zoom.addChangeListener(this);
    zoom.setName("zoom");
    p.addComponent(zoom              ,1,3,1,1,0,0);
    _zooml = new JTextField(Double.toString(zoom.getValue()/100),5);
    _zooml.setEditable(false);
    p.addComponent(_zooml            ,2,3,1,1,0,0);
    
    JEditorPane ep =new JEditorPane("text/html",description);
    ep.setEditable(false);
    JScrollPane esp = new JScrollPane(ep);
    esp.setPreferredSize(new Dimension(250, 280));
    p.addComponent(esp               ,0,4,3,1,1,1);

    add(p);
  }
  
  /**
   *changelistener for sliders
   */
  public void stateChanged(ChangeEvent e){
    JSlider slider = (JSlider) e.getSource();
    
    if (slider.getName() == "lambda"){
      _lambdal.setText(Double.toString(slider.getValue()/100.0));
    }else if(slider.getName() == "A"){
      _Al.setText(Double.toString(slider.getValue()));
    }else if(slider.getName() == "phi"){
      _phil.setText(Double.toString(slider.getValue()/100.0));
    }else if(slider.getName() == "zoom"){
      _zooml.setText(Double.toString(slider.getValue()/100.0));
    }else{
       System.err.println("Huh? Got non existant slider?: "+
	                  slider.getName()+" "+slider.getValue());
    }
  }
}
