import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> Wave<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Does all the paintingstuff and uses double buffering to avoid flicker
 */
public class Painting extends Canvas{
  Wave2D _wave = null;
  double _zoom = 0.0;

  /**
   * Constructor - Initializes the zoom with 1.0
   */
  public Painting(Wave2D wave){
    _wave = wave;
    _zoom = 1.0;
  }

  /**
   * sets the zoom (think: AffineTransform.scale())
   */
  public void setZoom(double zoom){
    _zoom = zoom;
  }

  /**
   * Draws coordinatesystem and wave into the offscreen buffer
   * it also calculates start and end points of the wave to fill the whole canvas
   */
  private void offpaint(Graphics g){
    Graphics2D g2 = (Graphics2D) g;
    g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setBackground(Color.black);
    g2.clearRect(0, 0, getWidth(), getHeight());

    //draw coordinates
    g2.setColor(Color.blue);
    g2.draw(new Line2D.Double(0,getHeight()/2,getWidth(),getHeight()/2));
    g2.draw(new Line2D.Double(getWidth()/2,0,getWidth()/2,getHeight()));
    g2.translate(getWidth()/2,getHeight()/2);  // set 0,0 to center
    g2.scale(1.0,-1.0);  //Flip screen

    g2.scale(_zoom,_zoom);
    
    //make the wave fill the whole screen:    
    try{
      //get transformed point at (0,0) and (width of the canvas,0);
      double coords[] = {0.0,0.0,getWidth(),0.0};
      g2.getTransform().inverseTransform(coords,0,coords,0,2);
      _wave.setStart(coords[0]/_wave.getScale());
      _wave.setEnd(coords[2]/_wave.getScale());
    }catch(NoninvertibleTransformException e){
      System.err.println("The transform matrix wasn't invertable how did this happen?");
      e.printStackTrace();
    }
    
    //draw the wave
    g2.setColor(Color.green);
    g2.draw(_wave);        

  }
  
  /**
   * Overwrites original update and removes the screnn clearing as we use
   * doublebuffering by using offpaint()
   */
  public void update(Graphics g){
    paint(g);
  }
  
  /**
   * initializes the offscreenbuffer, calls offpaint() and paints the buffer
   */
  public void paint(Graphics g){
    Image buffer;
    buffer =  createImage(getWidth(),getHeight());
    offpaint(buffer.getGraphics());
    g.drawImage(buffer,0,0,this);
  }
}
