import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> Wave<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Creates a Wave Shape
 */
public class Wave2D implements Shape{
  double _A;   //Amplitude
  double _lambda;  //Wavelength
  double _phi; //Phase
  double _scale; //used by PathIterator
  double _start; //draw from here
  double _end; //draw til here

  /**
   * empty Constructor - initializes the wave with the values of a normal
   * sinus curve
   */
  public Wave2D(){
    _A = 1;
    _lambda = 2*Math.PI;
    _phi = 0;
    _scale = 10.0;
    _start = 0.0;
    _end = 2*Math.PI;
  }

  /**
   * constructor - takes amplitude, wavelength and phaseshift
   */
  public Wave2D(double A, double lambda, double phi){
    _A = A;
    _lambda = lambda;
    _phi = phi;
    _scale = 10.0;
    _start = 0.0;
    _end = 2*Math.PI;
  }

  /**
   * constructor - takes amplitude, wavelength, phaseshift, start- and endpoint
   */
  public Wave2D(double A, double lambda, double phi, double start, double end){
    _A = A;
    _lambda = lambda;
    _phi = phi;
    _scale = 10.0;
    _start = start;
    _end = end;
  }
  
  /**
   * sets last point to draw
   */
  public void setEnd(double end){
    _end = end;
  }
  
  /**
   * gets last point to draw
   */
  public double getEnd(){
    return _end;
  }
  
  /**
   * sets first point to draw
   */
  public void setStart(double start){
    _start = start;
  }
  
  /**
   * gets first point to draw
   */
  public double getStart(){
    return _start;
  }
  
  /**
   * sets amplitude
   */
  public void setAmplitude(double A) {
    _A = A;
  }
  
  /**
   * sets wavelength
   */
  public void setWaveLength(double lambda) {
    _lambda = lambda;
  }
  
  /**
   * sets phaseshift
   */
  public void setPhase(double phi) {
    _phi = phi;
  }

  /**
   * sets scale - the wave is calculated in 0.1 steps to calculate the x-value of
   * the point where the actual amplitude is to be drawn the current index is multiplicated
   * by this value. It is set to 10 by default. A value of 0.0 is NOT allowed
   */
  public void setScale(double scale){
    _scale = scale;
    if (_scale == 0.0){
      System.err.println("Wave2D: Scale of 0.0 is not allowed! Set to 1.0");
      _scale = 1.0;
    }
  }
  
  /**
   * returns the actual scale
   */
  public double getScale(){
    return _scale;
  }
  
  /**
   * calculates the amplitude for x
   */
  public double calcWave(double x){
    return (double) _A*Math.sin(((2*Math.PI)/_lambda)*x+_phi);
  }

  /**
   * returns the bounding box
   */
  public Rectangle getBounds(){
    return new Rectangle((int) Math.ceil(_start),
                         (int) Math.ceil(Math.abs(_A)),
			 (int) Math.ceil(_end),
			 (int) Math.ceil(Math.abs(_A)) * -1);
  }
  
  /**
   * returns the bounding box
   */
  public Rectangle2D getBounds2D(){
      return (Rectangle2D) getBounds();
  }
  
  /**
   * the wave has no inside this returns always false
   */
  public boolean contains(double x,double y){
    return false;
  }

  /**
   * the wave has no inside this returns always false
   */
  public boolean contains(Point2D p){
    return false;
  }

  /**
   * the wave has no inside this returns always false
   */
  public boolean contains(double x,double y,double w,double h){
    return false;
  }

  /**
   * the wave has no inside this returns always false
   */
  public boolean contains(Rectangle2D r){
    return false;
  }

  /**
   * returns always true - this may be wrong?
   */  
  public boolean intersects(double x,double y,double w,double h){
    return true;
  }

  /**
   * returns always true - this may be wrong?
   */  
  public boolean intersects(Rectangle2D r){
    return true;
  }

  /**
   * returns new WaveIterator
   */  
  public PathIterator getPathIterator(AffineTransform at){
    return new WaveIterator(this,at);
  }

  /**
   * returns new WaveIterator
   */  
  public PathIterator getPathIterator(AffineTransform at,double flatness){
    return new WaveIterator(this,at);
  }

}


