import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> Wave<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Implements the PathIterator for Wave2D
 */
class WaveIterator implements PathIterator{
  double _index;
  Wave2D _w2d;
  AffineTransform _affine;

  /*
   * constructor
   */
  WaveIterator(Wave2D w2d, AffineTransform at){
    _w2d = w2d;
    _affine = at;
    _index = w2d.getStart();
  }
  
  /*
   * winding rule is not really needed as there is no "inside"
   */
  public int getWindingRule(){
    return WIND_EVEN_ODD;
  }
  
  /*
   * Pathtraversing finsihed?
   */
  public boolean isDone(){
    return (_index >  _w2d.getEnd());
  }
  
  /*
   * returns next Segment of the path
   */
  public void next(){
    _index = _index+0.1;
  };
  
  /*
   * returns current segments coordinates as float
   */
  public int currentSegment(float[] coords){
    if (isDone()) {
      throw new NoSuchElementException("line iterator out of bounds");
    }
    int type;
  
    coords[0] = (float) _index * (float) _w2d.getScale();
    coords[1] = (float) _w2d.calcWave(_index);
    
    if (_index == _w2d.getStart()){
      type = SEG_MOVETO;
    }else{
      type = SEG_LINETO;
    }
    
    if (_affine != null) {
      _affine.transform(coords, 0, coords, 0, 1);
    }
    return type;
  }

  /*
   * returns current segments coordinates as double
   */
  public int currentSegment(double[] coords){
    if (isDone()) {
      throw new NoSuchElementException("line iterator out of bounds");
    }
    int type;
  
    coords[0] = _index * _w2d.getScale();
    coords[1] = _w2d.calcWave(_index) ;
 
    if (_index == _w2d.getStart()){
      type = SEG_MOVETO;
    }else{
      type = SEG_LINETO;
    }
   
    if (_affine != null) {
      _affine.transform(coords, 0, coords, 0, 1);
    }
    return type;
  }
}
